/*
 EPSHeader

   File: binsrch.e
   Author: J. Kercheval
   Created: Tue, 09/17/1991  00:06:42
*/
/*
 EPSRevision History

   J. Kercheval  Tue, 09/17/1991  00:07:22  creation
   J. Kercheval  Sun, 09/22/1991  13:35:16  use current buffer for search
*/

/*
 * This file implements a simple recursive binary search and returns TRUE if
 * the element passed is present in the buffer and FALSE if not.  This module
 * assumes the buffer to search is the current buffer and that the current
 * buffer is sorted.
 *
 * This code is intended as a helper function for other module such as a tags
 * search or dictionary lookup, etc.
 *
 * This code is dedicated to the public domain with the caveat that Lugaru is
 * welcome to use this within their distribution source code which is
 * supplied with Epsilon.
 *
 *		John Kercheval
 *		15563 10th Ave NE
 *		Seattle, WA 98155-6209
 *		INTERNET: johnk@wrq.com
 *		Compu$erve: 72450,3702
 *      September 22, 1991
 */

#include <eel.h>

#ifndef BOOLEAN
#define BOOLEAN int
#define TRUE 1
#define FALSE 0
#endif


/*----------------------------------------------------------------------------
 *
 * binary_search_descend() is the recursive function which determines if the
 * current element is in the list.
 *
 ---------------------------------------------------------------------------*/

BOOLEAN binary_search_descend(elem, start, end, previous)
    char *elem;
    int start;
    int end;
    int previous;
{
    int cur_spot;
    int compare;
    char c, sc, *s;

    point = (start + end) / 2;

    /* goto next location to check */
    if (nl_reverse()) {
        point++;
    }

    if (previous == point) {
        nl_forward();
        cur_spot = point;
        previous = cur_spot;

        /* check for end condition after end */
        if (previous >= end) {
            point = end;
            return FALSE;
        }
    }
    else {
        cur_spot = point;
        previous = cur_spot;
    }

    /* check for end condition */
    if (start == end) {
        point = end;
        return FALSE;
    }

    /* do a binary search on the word buffer */
    s = elem;
    sc = *s;
    c = curchar();
    while (sc) {
        if (compare = charfcmp(c, sc)) {

            /* if the element is longer than the buffer line than it is
             * defined as being larger */
            if (c == '\n') {
                return binary_search_descend(elem, cur_spot, end, previous);
            }
            if (compare > 0)
                return binary_search_descend(elem, start, cur_spot, previous);
            else
                return binary_search_descend(elem, cur_spot, end, previous);
        }

        point++;
        s++;
        sc = *s;
        c = curchar();
    }

    /* if we reach here then the strings are the same if next c == '\n' and
     * elem in buffer is greater if not */
    if (c == '\n')
        return TRUE;
    else
        return binary_search_descend(elem, start, cur_spot, previous);
}


/*----------------------------------------------------------------------------
 *
 * do_binary_search() is merely a shell to store the old point and buffer and
 * to call the recursive search function with the initial values for its
 * parameters.
 *
 ---------------------------------------------------------------------------*/

BOOLEAN do_binary_search(elem)
    char *elem;
{
    point = size();
    to_begin_line();

    return binary_search_descend(elem, 0, point, -1);
}
